/**
 * 
 */
package com.b510.irregularverbs.client;

import java.util.Scanner;

import com.b510.irregularverbs.utils.CommonUtil;
import com.b510.irregularverbs.utils.IrregularVerbUtil;
import com.b510.irregularverbs.utils.PrintUtil;

/**
 * User input some commands in the console,then the system will receive<br> 
 * the command and process it, and display the result to user.<br>
 * for example: 
 * <li>all : show the all of the Irregular Verbs</li>
 * <li>pronunciation : the switch of the pronunciation.and the system default status<br>
 *     is not pronunciation.</li>
 * <li>The Irregular Verbs : user input the irregular verb,for example: "swim", "cost", "became".etc.</li>
 * @author Hongten
 * @created Feb 27, 2014
 */
public class Client {

	private Scanner inputStreamScanner;
	private boolean controlFlag = false;

	public static void main(String[] args) {
		Client client = new Client();
		client.listenConsoleAndHandleContent();
	}

	public Client() {
		inputStreamScanner = new Scanner(System.in);
	}

	/**
	 * Listen to the console and handle the content,what user inputed.
	 */
	public void listenConsoleAndHandleContent() {
		PrintUtil.printInfo(CommonUtil.DESCRIPTION);
		try {
			while (!controlFlag && inputStreamScanner.hasNext()) {
				processingConsoleInput();
			}
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			closeScanner();
		}
	}

	/**
	 * The system receive the content, and process it.
	 */
	private void processingConsoleInput() {
		String inputContent = inputStreamScanner.nextLine();
		if (!inputContent.isEmpty()) {
			if (pronounce(inputContent)) {
				IrregularVerbUtil.reLoadVerbs();
			} else {
				if (showAll(inputContent)) {
					return;
				}
				controlFlag = exit(inputContent);
				if (!controlFlag) {
					notExit(inputContent);
				}
			}
		}
	}

	/**
	 * When user input the command "all"(ignore case),the system will handle the command "all",<br>
	 * and display the all of the irregular verbs.
	 * @param inputContent the command "all"(ignore case)
	 * @return true, if user input the command "all"(ignore case);return false,if user input other command.
	 */
	private boolean showAll(String inputContent) {
		if (CommonUtil.ALL.equalsIgnoreCase(inputContent)) {
			IrregularVerbUtil.showAll();
			return true;
		} else {
			return false;
		}
	}

	/**
	 * When user input the command "pronunciation"(ignore case), the system will handle the command "pronunciation",<br>
	 * and reload all of the irregular verbs. and display the message "After loading vocabulary!".
	 * @param inputContent the command "pronunciation"(ignore case)
	 * @return true, if user input the command "pronunciation"(ignore case);return false, if user input other command.
	 */
	private boolean pronounce(String inputContent) {
		if (CommonUtil.PRONUNCIATION.equalsIgnoreCase(inputContent)) {
			IrregularVerbUtil.proFlag = !IrregularVerbUtil.proFlag;
			return true;
		} else {
			return false;
		}
	}

	private void notExit(String inputContent) {
		IrregularVerbUtil verbsLib = IrregularVerbUtil.getInstance();
		verbsLib.match(inputContent);
	}

	private boolean exit(String inputContent) {
		if (CommonUtil.EXIT.equalsIgnoreCase(inputContent)) {
			controlFlag = true;
			systemExit();
		}
		return controlFlag;
	}

	private void systemExit() {
		System.out.println(CommonUtil.EXIT_SYSTEM);
		System.exit(0);
	}

	private void closeScanner() {
		inputStreamScanner.close();
	}
}
