/**
 * 
 */
package com.b510.irregularverbs.utils;

import java.util.ArrayList;
import java.util.List;

import com.b510.irregularverbs.lib.VocabularyLib;
import com.b510.irregularverbs.vo.IrregularVerbsVO;

/**
 * The tool for the Irregular Verbs. Providing load and reload methods to get all irregular verbs.<br>
 * it also provide the match method to handle the content, which user input in the console.<br>
 * and display the result of matching.
 * @author Hongten
 * @created Feb 27, 2014
 */
public class IrregularVerbUtil {

	private static IrregularVerbUtil irregularVerbs = new IrregularVerbUtil();
	public static List<IrregularVerbsVO> verbs = new ArrayList<IrregularVerbsVO>();
	// The switch of the pronunciation
	public static boolean proFlag = false;

	/**
	 * Load all of the irregular verbs.<br>
	 * @return all of the irregular verbs.
	 */
	public static List<IrregularVerbsVO> loadIrregularVerbsVOs() {
		return VocabularyLib.getVocabularyLib();
	}

	private IrregularVerbUtil() {
	}

	public static IrregularVerbUtil getInstance() {
		if (verbs.size() <= 0) {
			loadIrregularVerbsVOs();
		}
		return irregularVerbs;
	}

	/**
	 * Reload all of the irregular verbs.
	 */
	public static void reLoadVerbs() {
		verbs = null;
		verbs = new ArrayList<IrregularVerbsVO>();
		loadIrregularVerbsVOs();
		PrintUtil.printInfo(CommonUtil.LOADIGN_VERBS);
		PrintUtil.printInfo(CommonUtil.LINE_THREE);
	}

	/**
	 * Processing the input content,and display the result for matching.
	 * @param input input content.
	 */
	public void match(String input) {
		long startTime = System.currentTimeMillis();
		List<IrregularVerbsVO> temp = new ArrayList<IrregularVerbsVO>();
		List<IrregularVerbsVO> mayBeVOs = new ArrayList<IrregularVerbsVO>();
		for (IrregularVerbsVO vo : verbs) {
			handleMatchWord(input, temp, vo);
			// match same word(s)
			input = input.toLowerCase();
			if (vo.getPrototype().startsWith(input) || vo.getPostTense().startsWith(input) || vo.getPostParticiple().startsWith(input)) {
				mayBeVOs.add(vo);
			}
		}
		PrintUtil.printInfo(CommonUtil.LINE_TWO);
		matchResult(temp, mayBeVOs);
		long endTime = System.currentTimeMillis();
		PrintUtil.printInfo(CommonUtil.SPEND + (endTime - startTime) + CommonUtil.SPEND_END);
		PrintUtil.printInfo(CommonUtil.LINE_THREE);
	}

	/**
	 * There have two results for matching.<br>
	 * <li>First : matched, then display the result of matched.</li>
	 * <li>Second : not matched, then display the related irregular verb(s)</li>
	 * @param matchedList
	 * @param relatedList
	 */
	private void matchResult(List<IrregularVerbsVO> matchedList, List<IrregularVerbsVO> relatedList) {
		if (matchedList.size() > 0) {
			loopMatchResult(matchedList);
		} else {
			PrintUtil.printInfo(CommonUtil.NO_MATCHED);
			if (relatedList.size() > 0) {
				PrintUtil.printInfo(CommonUtil.LINE_ONE);
				PrintUtil.printInfo(CommonUtil.ADVICE);
				loopMatchResult(relatedList);
			}
		}
	}

	/**
	 * Display the match result
	 * @param list
	 */
	private static void loopMatchResult(List<IrregularVerbsVO> list) {
		for (IrregularVerbsVO vo : list) {
			PrintUtil.printInfo(addBlank(vo.getPrototype().replace(CommonUtil.COMMA, CommonUtil.EMPTY))
					+ addBlank(vo.getPostTense().replace(CommonUtil.COMMA, CommonUtil.EMPTY))
					+ addBlank(vo.getPostParticiple().replace(CommonUtil.COMMA, CommonUtil.EMPTY)) + vo.getMean());
		}
	}

	/**
	 * @param input
	 * @param temp
	 * @param vo
	 */
	private void handleMatchWord(String input, List<IrregularVerbsVO> temp, IrregularVerbsVO vo) {
		if (proFlag) {
			if (input.equalsIgnoreCase(vo.getPrototype().split(CommonUtil.COMMA)[0]) || input.equalsIgnoreCase(vo.getPostTense().split(CommonUtil.COMMA)[0])
					|| input.equalsIgnoreCase(vo.getPostParticiple().split(CommonUtil.COMMA)[0])) {
				temp.add(vo);
			}
		} else {
			if (input.equalsIgnoreCase(vo.getPrototype()) || input.equalsIgnoreCase(vo.getPostTense()) || input.equalsIgnoreCase(vo.getPostParticiple())) {
				temp.add(vo);
			}
		}
	}

	/**
	 * Show all of the irregular verbs.
	 */
	public static void showAll() {
		if (verbs.size() <= 0) {
			loadIrregularVerbsVOs();
		}
		long startTime = System.currentTimeMillis();
		PrintUtil.printInfo(CommonUtil.LINE_TWO);
		loopMatchResult(verbs);
		long endTime = System.currentTimeMillis();
		PrintUtil.printInfo(CommonUtil.SPEND + (endTime - startTime) + CommonUtil.SPEND_END);
		PrintUtil.printInfo(CommonUtil.LINE_THREE);
	}

	/**
	 * Add the blank
	 * @param str
	 * @return
	 */
	private static String addBlank(String str) {
		int blank = CommonUtil.STR_SIZE - str.getBytes().length;
		for (int i = 0; i < blank; i++) {
			str += CommonUtil.BLANK;
		}
		return str.toString();
	}
}
